/* 
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File floppy.c : interface to BIOS floppy routines
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
 
#include <bios.h>

#include "z80.h" 
#include "floppy.h"

#define FLOPPY_ATTEMPTS	4
 

int
FloppyInit(int PCDrive)
{

	/* Initialise the floppy drive */
	
	int Status, iAttempt;	
        char TmpBuffer[512];

	iAttempt = 0; 
	do {
		Status  = biosdisk(_DISK_RESET,  PCDrive, 0, 
			  0, 0, 1, TmpBuffer);
                Status |= biosdisk(_DISK_STATUS, PCDrive, 0,
                          0, 0, 1, TmpBuffer);
		iAttempt++;
	} while (Status && (iAttempt < FLOPPY_ATTEMPTS));


	return Status;

}

int
FloppyRead(int PCDrive, int Side, int Track, int Sector, char *BufferPtr)
{
	/* Read a sector from floppy PCDrive into BufferPtr */
	/* Make a maximum of FLOPPY_ATTEMPTS to carry out   */
	/* the operation                                    */

	int Status, iAttempt;	

	iAttempt = 0; 
	do {
		Status = biosdisk(_DISK_READ, PCDrive, Side, 
			  Track, Sector, 1, BufferPtr);
		iAttempt++;
	} while (Status && (iAttempt < FLOPPY_ATTEMPTS));


	return Status;
}

int
FloppyWrite(int PCDrive, int Side, int Track, int Sector, char *BufferPtr)
{
	/* Write a sector from floppy PCDrive into BufferPtr */
	/* Make a maximum of FLOPPY_ATTEMPTS to carry out    */
	/* the operation                                     */

	int Status, iAttempt;	

	iAttempt = 0; 
	do {
		Status = biosdisk(_DISK_WRITE, PCDrive, Side, 
			  Track, Sector, 1, BufferPtr);
		iAttempt++;
	} while (Status && (iAttempt < FLOPPY_ATTEMPTS));
	return Status;
}

