/* 
 * ASCD : copyright (c) Aley Keprt 1998-2001
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File keyb_int.c - Public interface to keyboard interrupt handler for DOS 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * History :
 * Original Author : H. Hanemaayer (hhanemaa@cs.ruu.nl)
 * (Keyboard I/O based on showkey.c from kbd-0.84 package)
 * DOS port by Mihai Cartoaje ( moisemih@ift.ulaval.ca ).
 */


#ifndef KEYB_INT_H
#define KEYB_INT_H


#ifdef __cplusplus
extern "C" {
#endif


#define SCAN_UK_HASH        43  //=backslash
#define SCAN_UK_BACKSLASH   86  //=less

#define SCAN_ESCAPE		1
#define SCAN_1		2
#define SCAN_2		3
#define SCAN_3		4
#define SCAN_4		5
#define SCAN_5		6
#define SCAN_6		7
#define SCAN_7		8
#define SCAN_8		9
#define SCAN_9	 10
#define SCAN_0	 11

#define SCAN_MINUS			12
#define SCAN_EQUAL			13

#define SCAN_BACKSPACE		14
#define SCAN_TAB			15

#define SCAN_Q			16
#define SCAN_W			17
#define SCAN_E			18
#define SCAN_R			19
#define SCAN_T			20
#define SCAN_Y			21
#define SCAN_U			22
#define SCAN_I			23
#define SCAN_O			24
#define SCAN_P			25
#define SCAN_BRACKET_LEFT		26
#define SCAN_BRACKET_RIGHT		27

#define SCAN_ENTER			28

#define SCAN_LEFTCONTROL		29

#define SCAN_A			30
#define SCAN_S			31
#define SCAN_D			32
#define SCAN_F			33
#define SCAN_G			34
#define SCAN_H			35
#define SCAN_J			36
#define SCAN_K			37
#define SCAN_L			38
#define SCAN_SEMICOLON		39
#define SCAN_APOSTROPHE		40
#define SCAN_GRAVE			41

#define SCAN_LEFTSHIFT		42
#define SCAN_BACKSLASH		43

#define SCAN_Z			44
#define SCAN_X			45
#define SCAN_C			46
#define SCAN_V			47
#define SCAN_B			48
#define SCAN_N			49
#define SCAN_M			50
#define SCAN_COMMA	51
#define SCAN_PERIOD	52
#define SCAN_SLASH	53

#define SCAN_RIGHTSHIFT		54
#define SCAN_KEYPADMULTIPLY		55

#define SCAN_LEFTALT		56
#define SCAN_SPACE			57
#define SCAN_CAPSLOCK		58

#define SCAN_F1			59
#define SCAN_F2			60
#define SCAN_F3			61
#define SCAN_F4			62
#define SCAN_F5			63
#define SCAN_F6			64
#define SCAN_F7			65
#define SCAN_F8			66
#define SCAN_F9			67
#define SCAN_F10		68

#define SCAN_NUMLOCK		69
#define SCAN_SCROLLLOCK	70

#define SCAN_KEYPAD7		71
#define SCAN_CURSORUPLEFT		71
#define SCAN_KEYPAD8		72
#define SCAN_CURSORUP		72
#define SCAN_KEYPAD9		73
#define SCAN_CURSORUPRIGHT		73
#define SCAN_KEYPADMINUS		74
#define SCAN_KEYPAD4		75
#define SCAN_CURSORLEFT	75
#define SCAN_KEYPAD5		76
#define SCAN_KEYPAD6		77
#define SCAN_CURSORRIGHT		77
#define SCAN_KEYPADPLUS	78
#define SCAN_KEYPAD1		79
#define SCAN_CURSORDOWNLEFT		79
#define SCAN_KEYPAD2		80
#define SCAN_CURSORDOWN	80
#define SCAN_KEYPAD3		81
#define SCAN_CURSORDOWNRIGHT	81
#define SCAN_KEYPAD0		82
#define SCAN_KEYPADPERIOD		83

#define SCAN_LESS			86

#define SCAN_F11			87
#define SCAN_F12			88

#define SCAN_KEYPADENTER		96
#define SCAN_RIGHTCONTROL		97
#define SCAN_CONTROL		97
#define SCAN_KEYPADDIVIDE		98
#define SCAN_PRINTSCREEN		99
#define SCAN_RIGHTALT		100
#define SCAN_BREAK			101	/* Beware: is 119     */
#define SCAN_BREAK_ALTERNATIVE	119	/* on some keyboards! */

#define SCAN_HOME			102
#define SCAN_CURSORBLOCKUP		103	/* Cursor key block */
#define SCAN_PAGEUP			104
#define SCAN_CURSORBLOCKLEFT	105	/* Cursor key block */
#define SCAN_CURSORBLOCKRIGHT	106	/* Cursor key block */
#define SCAN_END			107
#define SCAN_CURSORBLOCKDOWN	108	/* Cursor key block */
#define SCAN_PAGEDOWN		109
#define SCAN_INSERT			110
#define SCAN_DELETE			111

#define KEY_EVENTRELEASE		0
#define KEY_EVENTPRESS			1


void ProcessKeyBuf(void); //zpracuje data v bufferu klavesnice
typedef void (*__KeyboardHandler)(int scancode, int press);
int  KeyboardInit( __KeyboardHandler Handler );
void KeyboardClose( void );
int  KeyboardUpdate( void );


#ifdef __cplusplus
}
#endif


#endif
