/*
 * ASCD : copyright (c) Aley Keprt 1998-2001
 * SimCoupe : Copyright (c) Allan Skillman 1996
 *
 * File time_int.c - Timer interrupt handler
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdio.h>
#include <dos.h>
#include <dpmi.h>
#include <go32.h>

#include "intdef.h"


static int TimerInstalled = False;

static int *TimerFlagPtr;
static int TimerTimeOut;
static int TimerTimeOutCounter;
static long BiosCounter;                 /* keep BIOS time up to date */
static long TimerDelay;                  /* how long between interrupts */
static int TimerSemaphore = False;       /* reentrant interrupt? */
static int old_hz;


static _go32_dpmi_seginfo wrapper, oldHandler, newHandler;


static void TimerSetRate(long time) {
/*  Sets the delay time for PIT channel 1 in cycle mode. */
  outportb(0x43, 0x34);
  outportb(0x40, time & 0xff);
  outportb(0x40, time >> 8);
}


static int TimerInt(void) {
/*  Hardware level timer interrupt (int 8) handler. */
  int bios = False;

  /* update bios time */
  BiosCounter -= TimerDelay;
  if(BiosCounter <= 0) {
    BiosCounter += 0x10000;
    bios = True;
  }

  /* Deal with the Countdown Timer */
  if(TimerTimeOutCounter-- <= 0) {
    TimerTimeOutCounter += TimerTimeOut;
    *TimerFlagPtr = True;
  }

#if 0
  if(!bios) {
    ENABLE();
    outportb(0x20, 0x20);      /* ack. the interrupt */
  }

  if(!bios) DISABLE();
#endif

/*   outportb(0x20, 0x20);  */    /* ack. the interrupt */

  return bios;
}
END_OF_FUNCTION(TimerInt);


int TimerInstall(int CountsPerSec, int *FlagPtr)
{
  int x;

  old_hz = CountsPerSec;

  if(TimerInstalled) return -1;

  LOCK_VARIABLE(wrapper);
  LOCK_VARIABLE(oldHandler);
  LOCK_VARIABLE(newHandler);
  LOCK_VARIABLE(TimerFlagPtr);
  LOCK_VARIABLE(TimerTimeOut);
  LOCK_VARIABLE(TimerTimeOutCounter);
  LOCK_VARIABLE(BiosCounter);
  LOCK_VARIABLE(TimerDelay);
  LOCK_VARIABLE(TimerSemaphore);
  LOCK_FUNCTION(TimerInt);

  BiosCounter = 0x10000;

  TimerDelay = BPS_TO_TIMER(200);

  TimerFlagPtr = FlagPtr;
  TimerTimeOut = BPS_TO_TIMER(CountsPerSec) / TimerDelay;
  TimerTimeOutCounter = TimerTimeOut;

  _go32_dpmi_get_protected_mode_interrupt_vector(TIMER_INT, &oldHandler);

  DISABLE();

#if 1
  newHandler.pm_offset = (int)TimerInt;
  newHandler.pm_selector = _go32_my_cs();
  _go32_dpmi_chain_protected_mode_interrupt_vector(TIMER_INT, &newHandler);
#else
  wrapper.pm_offset = (int)TimerInt;
  wrapper.pm_selector = _go32_my_cs();
  _go32_dpmi_allocate_iret_wrapper(&wrapper);

  newHandler.pm_offset=wrapper.pm_offset;
  newHandler.pm_selector=wrapper.pm_selector;
  _go32_dpmi_set_protected_mode_interrupt_vector(TIMER_INT, &newHandler);
#endif

   /* sometimes it doesn't seem to register if we only do this once... */
   for(x=0; x<4; x++) {
#if 1
     TimerSetRate(TimerDelay);
#endif
   }

   ENABLE();

   TimerInstalled = True;
   return 0;
}


void TimerQuit(void) {
   if (!TimerInstalled) return;

   DISABLE();

   TimerSetRate(0);
   _go32_dpmi_set_protected_mode_interrupt_vector(TIMER_INT, &oldHandler);
   TimerSetRate(0);

   ENABLE();

#if 0 //wrapper isn't used
   _go32_dpmi_free_iret_wrapper(&wrapper);
#endif

   TimerInstalled = False;
}


void TimerReinstall(void) {
  if(old_hz) TimerInstall(old_hz,TimerFlagPtr);
}
