/*
 * ASCD : copyright (c) Aley Keprt 1998-2001
 *
 * File fux.c - .fux file handling (Fuxoft emulator snapshots)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdio.h>
#include "zlib.h"

#include "z80.h"
#include "z80_snap.h"
#include "mem.h"


typedef unsigned char byte;


typedef struct {
  byte res,i;
  byte iffpv,r;
  byte iy[2],ix[2],hl1[2],de1[2],bc1[2],f1,a1;
  byte hl[2],de[2],bc[2],sp[2];
} FUX_header;


//-----------------------------------------------------------------------------------


#define mw(reg) (r.reg=hdr.reg[0]+hdr.reg[1]*256)
#define mb(h,l,hl) (r.h=hdr.hl[1],r.l=hdr.hl[0])


int load_fux_gzf(gzFile *f) {
//
//internal function
//load .fux snapshot
//
//return: error flag
//
  FUX_header hdr;
  Z80_Regs r;

  if(gzread(f,&hdr,24)!=24) return -1; //error

  //initialize registers
  mb(b,c,bc);
  mb(d,e,de);
  mb(h,l,hl);

  mw(ix);
  mw(iy);
  mw(sp);

  r.a1=hdr.a1;
  r.f1=hdr.f1;

  mb(b1,c1,bc1);
  mb(d1,e1,de1);
  mb(h1,l1,hl1);

  r.iff1=r.iff2=(hdr.iffpv!=0);

  r.i=hdr.i;
  r.r=hdr.r;

  r.im=(r.i<0x40) ? 2/*im1*/ : 3/*im2*/;

  //load 48k body
  if(gzread(f,realMem[5],16384)!=16384) return -1; //error
  if(gzread(f,realMem[2],16384)!=16384) return -1; //error
  if(gzread(f,realMem[0],16384)!=16384) return -1; //error

  //initialize ports
  out_byte(BORDER_PORT,0);

  set_emulation_mode(EM_48k);

  //af and pc are on the stack
  r.f=read_byte(r.sp);
  r.sp++;
  r.a=read_byte(r.sp);
  r.sp++;
  r.pc=read_word(r.sp);
  r.sp+=2;

  //important! NMI routine must be replaced by an empty one!
  memset(&realMem[ROM0][0x68],0,8);

  set_registers(&r);

  return 0;
}

#undef mw
#undef mb


//-----------------------------------------------------------------------


int load_fux(const char *fname) {
//
//load .fux snapshot
//
//return: error flag
//
  gzFile *gzf;
  int r;

  gzf=gzopen(fname,"rb");
  if(!gzf) return -1;

  r=load_fux_gzf(gzf);
  gzclose(gzf);
  return r; //return error flag
}

