/* 
 * ASCD : copyright (c) Aley Keprt 1998-2001
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File main.c - initial setup etc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#define Z80_MAIN

#include <stdio.h>
#include <signal.h>
#include <time.h>
#include <crt0.h>
#include <conio.h>
#include <unistd.h>

#include "mem.h"
#include "resource.h"
#include "util.h"
#include "fdi.h"
#include "intrface.h"
#include "audio.h"
#include "speaker.h"
#include "video.h"
#include "audio.h"
#include "tape.h"
#include "kbd.h"

#ifdef USE_AIR
#include "AirClient.h"
#endif


//removes wildcard expansion of DJGPP
char **__crt0_glob_function(char *_arg) {return 0;}

//don't remove quotes and other characters from filenames
int _crt0_startup_flags = _CRT0_FLAG_KEEP_QUOTES;


double elapsed_time;   //cas v sekundach, jak dlouho bezela emulace
double time_ratio;     //pomer casu sam/real (1.0=100%)

static void got_sig(int signo);
static long start_time; //clock() pri spusteni emulatoru - pro vypocet rychlosti emulace

static char cwd[1000];


void timer_stats(void);
void eject_disks(void);


static void got_sig(int signo) {
  Quit(0);
}


void timer_stats(void) {
  extern long n_frames;
  double samtime=n_frames/50.0;
  if(elapsed_time==0) elapsed_time=(double)(clock()-start_time)/CLOCKS_PER_SEC;
  time_ratio=samtime/elapsed_time;

  simcoupe_mesg(SIMCOUPE_INFO,"Sam time:  %.2f sec",samtime);
  simcoupe_mesg(SIMCOUPE_INFO,"Real time: %.2f sec",elapsed_time);
  simcoupe_mesg(SIMCOUPE_INFO,"Ratio:     %.2f%%",time_ratio*100);
}


void eject_disks(void) {
  eject_disk(1);
  eject_disk(2);
}


void RestoreCwd(void) {
//
//obnovi puvodni adresar
//
  chdir(cwd);
}


#define try(cmd) simcoupe_mesg(SIMCOUPE_INFO,#cmd); cmd


int main(int argc, char **argv)
{
  while(kbhit()) getch();

  simcoupe_mesg(SIMCOUPE_INFO,"ASCD Initialization:");

  getcwd(cwd,sizeof(cwd));
  OnQuit(RestoreCwd,"RestoreCwd()");

  get_exe_path(argv[0]);

  //toto se musi delat na zacatku, protoze to muze ukoncit program
  init_cfg(argc,argv);

  //turn off cursor emulation on arrows, if arrows are already used
  if(cfg.cursor==0 && (cfg.kempston==0 || cfg.sinclair1>=0 || cfg.sinclair2>=0)) {
    cfg.cursor=-1;
  }

  /* Make sure we can't be killed off while initialising stuff,
   * or we might end up leaving shared memory segments lying about */
  signal(SIGINT,  SIG_IGN);  //Ctrl+C & Ctrl+Break
  signal(SIGQUIT, SIG_IGN);  //quit signal - has no effect
#ifndef DJGPP
  signal(SIGHUP,  SIG_IGN);  //unused
  signal(SIGTERM, SIG_IGN);  //unused
#endif

  init_log();

  KbdInit();
  EventsInit();
  VideoInit();
  JoyInit();
  try(eject_disks());

  //playback nastartuje pred nahratim souboru z disku
  //taky pred inicializaci pameti emulatoru
#ifdef USE_AIR
  if(cfg.record[0] || cfg.playback[0]) OnQuit(Air_close,"Air_close()");
  if(cfg.playback[0]) airplayback(cfg.playback);
#endif

  try(init_memory());
  try(init_ports());
  set_emulation_mode(cfg.zxmode);

  if(GetCfg(disk1)[0]) insert_disk(GetCfg(disk1),1);
  if(GetCfg(disk2)[0]) insert_disk(GetCfg(disk2),2);

  signal(SIGINT, got_sig);
  signal(SIGQUIT,got_sig);
#ifndef DJGPP
  signal(SIGHUP, got_sig);
  signal(SIGTERM,got_sig);
#endif

  OnQuit(close_tap,"close_tap()");

  /* Insert timing stats function into Quit sequence */
  OnQuit(timer_stats,"timer_stats()");

  /* Better make sure the disk images are properly closed */
  OnQuit(eject_disks,"eject_disks()");

  SpeakerOn();
  OnQuit(SpeakerOff,"SpeakerOff()");

  if(cfg.audio) AudioInit();

  //record nastartuje az po nahrati souboru z disku
  //taky co nejpozdeji, aby nedoslo ke zpozdeni v casomire
#ifdef USE_AIR
  if(cfg.record[0]) airrecord(cfg.disk1,cfg.record,cfg.Scode);
#endif

  /* Start timing the emulator */
  startwatch(0);
  start_time=clock();

  simcoupe_mesg(SIMCOUPE_INFO,"");
  simcoupe_mesg(SIMCOUPE_INFO,"Emulator is running");
  simcoupe_mesg(SIMCOUPE_INFO,"");

  emulator_main_loop();

  return 0;
}

