/* 
 * ASCD : copyright (c) Aley Keprt 1998-2001
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File mem.h - definitions and inline memory i/o routines.
 *
 * Intel 80x86 assembly optimisations Copyright (c) 1996 Allan Skillman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef __MEM_H
#define __MEM_H


#include <string.h>

#include "samio.h"
#include "dtbl.h"
#include "resource.h"


void set_emulation_mode(EM mode);

void get_exe_path(const char *exe);
int load_rom_image(char *image, int page);

void page_in(int bank, int page, int readonly);


/* 16K memory pages */
#define SAM_PAGESIZE 0x4000

/* Constants related to memory organisation */

#define NMAINPAGES	32	/* Number of pages in the SAM's main memory (512K) */
#define NEXPAGES	64	/* Number of pages in 1Mb of external memory */ 
#define MAXEXPAGES	4       /* Maximum number of 1Mb external memory allowed */

/* The Roms always go at the end of the memory */

#define ROM0   		(NMAINPAGES+(NEXPAGES*MAXEXPAGES))
#define ROM1   		(ROM0+1)
#define TOTAL_NO_PAGES	(ROM1+1)

#define SCRATCH_PAGE	0xffff


extern uns8 *theRdMemory[];
extern uns8 *theWrMemory[];
extern int real_pagenos[];
extern uns8 *realMem[];
extern uns8 *scratchMem[];

extern char exe_path[];


/* Map a 16 bit address through the memory indirection - allows fast paging */
#define BANK(addr)  ((addr) >> 14)
#define RPAGE(addr) (real_pagenos[BANK(addr)])
#define MEM_R(addr) (theRdMemory[BANK(addr)][(addr) & 0x3fff])
#define MEM_W(addr) (theWrMemory[BANK(addr)][(addr) & 0x3fff])
#define READ_ONLY(addr) ((RPAGE(addr) >= ROM0) || (!((addr) & 0xc000) && LMPR_WPRAM))


void init_memory(void);


//---------------------------------------------------------------------------


static inline uns8 read_byte(uns16 addr) {
	return MEM_R(addr);
}


//---------------------------------------------------------------------------


static inline uns16 read_word(uns16 addr) {
	return MEM_R(addr) + (MEM_R((uns16)(addr+1)) << 8);
}


//---------------------------------------------------------------------------


static void write_to_screen_vmpr0(uns16 addr) {

	/* from video.c */
	extern uns8 LineNeedsUpdating[];

	int AttrLine;

	addr &= 0x3fff;

  switch (VMPR_SCREEN_MODE) {
           
  case MODE_1:
    if(addr<6144) {
			LineNeedsUpdating[dtbl_y[addr]]=True;
		} else if(addr<6912) {
      AttrLine = ((addr-6144) & 0xffe0)>>2;
			memset(&LineNeedsUpdating[AttrLine],True,8);
		}
		break;

  case MODE_2:
    if(addr<0x3800) LineNeedsUpdating[(addr & 0x1fff)>>5]=True;
    break;

  case MODE_3:
  case MODE_4:
    LineNeedsUpdating[addr>>7] = True;
    break;
	}
}  


//---------------------------------------------------------------------------


static inline void write_to_screen_vmpr1(uns16 addr) {

	/* from video.c */
	extern uns8 LineNeedsUpdating[];

	addr&=0x3fff;
	if(addr<0x2000) LineNeedsUpdating[128+(addr>>7)] = True;
}


//---------------------------------------------------------------------------


static inline void write_byte(uns16 addr, uns8 val) {

	MEM_W(addr)=val;

  /* only do a screen write when necessary */
  if(RPAGE(addr)==VMPR_SCREEN_PAGE) {
    write_to_screen_vmpr0(addr);
  } else if(VMPR_SCREEN_MODE>MODE_2 &&
    RPAGE(addr)==((VMPR_SCREEN_PAGE+1)&0x1f)) {
    write_to_screen_vmpr1(addr);
  }
}


//---------------------------------------------------------------------------


static inline void write_word(uns16 addr, uns16 val) {

	MEM_W(addr)=(uns8)val;

	/* only do a screen write when necessary */
	if (RPAGE(addr) == VMPR_SCREEN_PAGE)
 		write_to_screen_vmpr0(addr);
 	else if ((VMPR_SCREEN_MODE > MODE_2)  &&
	         (RPAGE(addr) == ((VMPR_SCREEN_PAGE + 1) & 0x1f)))
	    	write_to_screen_vmpr1(addr);

	addr++;

	MEM_W(addr)=val>>8;

	/* only do a screen write when necessary */
  /* aley: required only when addr is now 32-bytes aligned */
  /* because it is on the same line otherwise */
  if(!(addr&31)) {
	  if(RPAGE(addr) == VMPR_SCREEN_PAGE) {
      write_to_screen_vmpr0(addr);
    }	else if((VMPR_SCREEN_MODE > MODE_2)  &&
	            (RPAGE(addr) == ((VMPR_SCREEN_PAGE + 1) & 0x1f))) {
    	write_to_screen_vmpr1(addr);
    }
  }
}


#undef BANK
#undef RPAGE
#undef MEM_R
#undef MEM_W
#undef READ_ONLY


#undef INTEL_ASM

#endif

