/* 
 * ASCD : copyright (c) Aley Keprt 1998-2001
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * File samio.c : I/O hardware emulation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdio.h>
#include <stdlib.h>

#include "mem.h"
#include "resource.h"
#include "util.h"
#include "version.h"
#include "mouse.h"
#include "fdi.h"
#include "printer.h"
#include "speaker.h"
#include "video.h"
#include "driver.h"
#include "ay8910.h"
#include "saa1099.h"


uns8 keyports[10];	/* 8 rows of keys (+ 1 row for unscanned keys) */


int ClutChangeInLine;
int BorderChangeInLine;
int VmprChangeInLine;
#ifdef USE_BORDER_EFFECTS
int BorderChangeInInstruction;
#endif

extern int totallines;
extern int toplines;
extern int LineCycleCounter;


char sampal[16]; //current palette

static int printl_data,printl_stat;

extern int LineNo;
extern int SAMPointerMode;
extern unsigned long pixels[]; /* 128 Sam colours */


/* The SAM ports */
int vmpr,hmpr,lmpr;
int midi;
int border;
uns8 clut[16];
int keyboard;
int status_reg;
int line_int;
int clock_reg;
int hepr,lepr;
int attr_reg;    //0xff
int beeperbit;
int kjport;      //kempston joystick port (0x31)
int fuller;      //fuller joystick (0x7f)


//---------------------------------------------------------------------------


void set_zx_palette(void) {
  int i;
  for(i=0;i<16;i++) clut[i]=(uns32)pixels[i | (i & 0x07)<<4];
  clut[8]=0; //fixed by Aley
}


//---------------------------------------------------------------------------


void init_ports(void)
{
	int i;

	vmpr = 0x81;
	hmpr = 0x01;
	lmpr = 0x1f;
	midi = 0x00;	
	border = 0x00;

  kjport = 0xe0;
  fuller=0xff;

	for(i=0;i<16;i++) clut[i]=(uns32)pixels[i | (i & 0x07)<<4];
  clut[8]=0; //ZX palette fixed by Aley

	keyboard = 0x5f;
	status_reg = 0xff;
	
	line_int = 0xff;
        	
  clock_reg = 0x00;

	lepr = 0x00;
	hepr = 0x00;
	attr_reg = 0xff;
}


//---------------------------------------------------------------------------


static inline void out_hepr(uns8 val)
{
	int ExternalPage;

  hepr=val;

/* If somebody tries to access non existent external memory, page in the 
   ROM in sections C or D. This means that read-write tests on the
   memory will fail - as required */

  if(HMPR_MCNTRL) {
    if((ExternalPage=NMAINPAGES+val) >= ROM0-(cfg.extmem<<6))
      page_in(3,ExternalPage,0);
    else
      page_in(3,ROM0,1);
	}
}


//---------------------------------------------------------------------------


static inline void out_lepr(uns8 val)
{
	int ExternalPage;

  lepr=val;

/* If somebody tries to access non existent external memory, page in the 
   ROM in sections C or D. This means that read-write tests on the
   memory will fail - as required */

 	if(HMPR_MCNTRL) {
		if((ExternalPage=NMAINPAGES+val) >= ROM0-(cfg.extmem<<6))
      page_in(2,ExternalPage,0);
    else
      page_in(2,ROM0,1);
	}
}


//---------------------------------------------------------------------------


inline void out_vmpr(uns8 val) {

  vmpr=(val&0x7f)|0x80 ;

  switch(VMPR_SCREEN_MODE) {
  case MODE_1: VideoSetMode(1);break;
  case MODE_2: VideoSetMode(2);break;
  case MODE_3: VideoSetMode(3);break;
  case MODE_4: VideoSetMode(4);break;
  }

  if(!BORD_SOFF) VmprChangeInLine=LineNo+1;
}


//---------------------------------------------------------------------------


inline void out_hmpr(uns8 val)
{
  hmpr = val;
        
	if (LMPR_ROM1) 
	   page_in(3,ROM1,True);
	else
	   page_in(3,(HMPR_PAGE + 1)& 0x1f, False);
  
	page_in(2, HMPR_PAGE, False);

   
	if (HMPR_MCNTRL) {
		out_hepr(hepr);
		out_lepr(lepr);
	}
}


//---------------------------------------------------------------------------


static inline void out_lmpr(uns8 val)
{
  lmpr = val;
        
	if (LMPR_RAM0)
	   page_in(0, LMPR_PAGE, LMPR_WPRAM);
	else
	   page_in(0, ROM0, True);

        page_in(1, (LMPR_PAGE + 1) & 0x1f, False);

        if (LMPR_ROM1)
           page_in(3, ROM1, True);
        else
           page_in(3, (HMPR_PAGE + 1) &0x1f, False);
           
}


//---------------------------------------------------------------------------


static inline void out_clut(uns16 port, uns8 val)
{
	uns32 old_val = clut[port & 0x0f];
  
	if (old_val != (uns32)pixels[val & 0x7f]) {
		clut[port & 0x0f] = (uns32)pixels[val & 0x7f];
//Aley'99-begin
    sampal[port&0xf]=val&0x7f;
//Aley'99-end
	 	if(!BORD_SOFF) {
			ClutChangeInLine=LineNo+1;
#ifdef USE_BORDER_EFFECTS
			BorderChangeInInstruction=True;
#endif
		}
	}
}


//---------------------------------------------------------------------------


void out_zxpage(int value) {
/*
  strankovani pameti zxs 128k
*/
  //D0-D2 = RAM page 3
  page_in(3,value&7,False);

  //D3 = video (5/7)
  {
    int vmpr_val=(value&8) ? 7 : 5;
    if((vmpr&0x7f)!=vmpr_val) out_vmpr(vmpr_val);
  }

  //D4 = ROM select
  page_in(0, (value&16) ? ROM1 : ROM0, True);

  //D5 = 48k lock
  if(value&32) {
//    page_in(
/*
      page_in(3, ROM1, True);
      else
      page_in(3, (HMPR_PAGE + 1) &0x1f, False);
*/
  }
}


//---------------------------------------------------------------------------


void out_byte(uns16 port,uns8 val) {

  if(GetCfg(zxmode)) {

    //ZXS mode

    if(!(port&1)) {

      //254 (A0) = border & speaker

     	if((val&7)!=border) {
        BorderChangeInLine=LineNo+1;
    	  border=val&7;
      }

      if((val&0x10) != beeperbit) {
        beeperbit=(val>>4)&1;
        if(GetCfg(use_speaker)) SpeakerPlay(beeperbit);
      }

    } else if(!(port&2)) {
      //A2=ZX Spectrum 128
      //detekuje se jen podle bitu A14-A15 (horni 2 bity)
      switch(port>>14) {

      case 1:
        out_zxpage(val);
        break;

      case 3:
        //AY reg
        AY8910Write(0,0,val);
        break;

      case 2:
        //AY data
        AY8910Write(0,1,val);
        break;

      case 0:
        //+3 regs - not supported
        simcoupe_mesg(SIMCOUPE_INFO,"[+3] out 0x%04x,%i",port,val);
        break;

      default:
        simcoupe_mesg(SIMCOUPE_INFO,"out 0x%04x,%i",port,val);
      }
    }
    return;
  }


	switch(port&0xff) {

	/* border color and speaker control */
  case BORDER_PORT:

   	if(BORD_COL(border)!=BORD_COL(val)) {
      BorderChangeInLine=LineNo+1;
#ifdef USE_BORDER_EFFECTS
      BorderChangeInInstruction=1;
#endif
    }

  	if(BORD_SOFF && !(val&0x80)) {
      //finally found the bug and fixed by Aley'01!
    	//display turned back on - we must update whole screen
      BorderChangeInLine=LineNo+1;
      ClutChangeInLine=LineNo+1;
      VmprChangeInLine=LineNo+1;
    }

    if((val&0x10) != beeperbit) {
      beeperbit=(val>>4)&1;
      if(GetCfg(use_speaker)) SpeakerPlay(beeperbit);
    }

    border=val;
   	keyboard=(keyboard&0x7f)|BORD_SOFF;
   	break;
	    
  case VMPR_PORT:
  	if((vmpr&0x7f) != (val&0x7f)) out_vmpr(val);
   	break;

  case HMPR_PORT:
   	if(hmpr!=val) out_hmpr(val);
   	break;

  case LMPR_PORT:
   	if(lmpr!=val) out_lmpr(val);
   	break;

  case CLUT_BASE_PORT:
    out_clut(port>>8,val);
    break;
        	
  /* External memory */
  case HEPR_PORT:
    out_hepr(val);
    break;

  case LEPR_PORT:
    out_lepr(val);
    break;
 	
  case LINE_PORT:
    line_int = val;
    break;

  case SOUND_DATA:
    //controlled by A8
    if(port&256) {
      saa1099_control_port_w(0,0,val);
    } else {
      saa1099_write_port_w(0,0,val);
    }
    break;

  /* Printer IO - by Aley Keprt*/
  case PRINTL1_STAT:
    val&=1;
    if(val==0 && printl_stat==1) PrinterWrite(0,printl_data);
    printl_stat=val;
    break;

  case PRINTL1_DATA:
    printl_data=val;
    break;
	}

  /* Disk IO */
  if(((port&0xff) >= DISK1_PORT && (port&0xff) < DISK1_PORT+8) ||
     ((port&0xff) >= DISK2_PORT && (port&0xff) < DISK2_PORT+8))
   	fdi_out(port,val);

#ifdef PSEUDO_IO
	if(port==PSEUDO_STDOUT)	putc((int)val,stdout);
	else if(port==PSEUDO_STDERR) putc((int)val,stderr);
#endif
}


//---------------------------------------------------------------------------


uns8 in_byte(uns16 port) {

	int res = 0x1f;
	int highbyte;

  if(GetCfg(zxmode)) {

    //ZX mode
    if((port&0xff)==ATTR_PORT) {
      return attr_reg ? attr_reg : read_attr();
    }

    if(!(port&1)) {

      //254 (A0) = keyboard & ear
		  highbyte=port>>8;

		  if(!(highbyte&0x80)) res &= keyports[7]&0x1f;
		  if(!(highbyte&0x40)) res &= keyports[6]&0x1f;
		  if(!(highbyte&0x20)) res &= keyports[5]&0x1f;
		  if(!(highbyte&0x10)) res &= keyports[4]&0x1f;
		  if(!(highbyte&0x08)) res &= keyports[3]&0x1f;
		  if(!(highbyte&0x04)) res &= keyports[2]&0x1f;
		  if(!(highbyte&0x02)) res &= keyports[1]&0x1f;
		  if(!(highbyte&0x01)) res &= keyports[0]&0x1f;
		  return res|0xa0; //bits5&7=1, bit6(ear)=0
    }

    if(!(port&0xe0)) {
      //31 = kempston joystick (A7+A6+A5)
      return kjport;
    }

    //AY
    if(port==0xfffd) return AY8910Read(0);

    //fuller
    if(port==0x7f) return fuller;

    return 0xff;
  }

  //Sam mode

  if((port&0xff)<0x10) return 0;
/*
  if((port&0xff)<=0x1f) {
    //31 = kempston joystick
    return kjport;
  }
*/
  switch(port&0xff) {

  case ATTR_PORT:
    return attr_reg;

  case KEYBOARD_PORT:  /* keyboard 1 / mouse */
		highbyte = port >> 8;
    if (highbyte ==  0xff)  {
     	res = keyports[8] & 0x1f;
			if (SAMPointerMode && (res == 0x1f)) {
				res = (int)(sam_mouse_in() & 0x1f);
			}
		}

		if(!(highbyte & 0x80)) res &= keyports[7] & 0x1f;
		if(!(highbyte & 0x40)) res &= keyports[6] & 0x1f;
		if(!(highbyte & 0x20)) res &= keyports[5] & 0x1f;
		if(!(highbyte & 0x10)) res &= keyports[4] & 0x1f;
		if(!(highbyte & 0x08)) res &= keyports[3] & 0x1f;
		if(!(highbyte & 0x04)) res &= keyports[2] & 0x1f;
		if(!(highbyte & 0x02)) res &= keyports[1] & 0x1f;
		if(!(highbyte & 0x01)) res &= keyports[0] & 0x1f;
		return (keyboard = (res | (keyboard & 0xe0)));

  case STATUS_PORT:  /* keyboard  2 */
 		highbyte = port >> 8;
 		res = 0xe0;
 		if(!(highbyte & 0x80)) res &= keyports[7] & 0xe0;
 		if(!(highbyte & 0x40)) res &= keyports[6] & 0xe0;
 		if(!(highbyte & 0x20)) res &= keyports[5] & 0xe0;
 		if(!(highbyte & 0x10)) res &= keyports[4] & 0xe0;
 		if(!(highbyte & 0x08)) res &= keyports[3] & 0xe0;
 		if(!(highbyte & 0x04)) res &= keyports[2] & 0xe0;
 		if(!(highbyte & 0x02)) res &= keyports[1] & 0xe0;
 		if(!(highbyte & 0x01)) res &= keyports[0] & 0xe0;
 		return (res&0xe0) | (status_reg&0x1f);

 	/* Banked memory management */

  case VMPR_PORT:
   	return vmpr;

  case HMPR_PORT:
   	return hmpr;

  case LMPR_PORT:
   	return lmpr;

  case CLOCK_PORT:
    return clock_reg;

  case LPEN_PORT:
    if(port&0x100) {
      //hpen=y
      return (LineNo>=toplines && LineNo<toplines+192) ? LineNo-toplines : 192;
    } else {
      //lpen=x (0..63)
      int x=LineCycleCounter>>2;
      return (x<64 ? x : 63) << 2;
    }
    break;

  /* External memory */
  case HEPR_PORT:
		return hepr ;

  case LEPR_PORT:
    return lepr;

  /* Printer IO */
  case PRINTL1_STAT:
    if(!PrinterStatus(0)) return 0x80;
	}

  if(!(port&0xe0)) {
    //31 = kempston joystick (A7+A6+A5)
    return kjport;
  }

  /* Disk IO */
  if(((port&0xff) >= DISK1_PORT && (port&0xff) < DISK1_PORT + 8) ||
     ((port&0xff) >= DISK2_PORT && (port&0xff) < DISK2_PORT + 8))
   	return(fdi_in(port));

#ifdef PSEUDO_IO
	if(port==PSEUDO_STDIN) return (uns8)getchar();
	else
#endif
	return 0xff;
}

