/* 
 * ASCD : copyright (c) Aley Keprt 1998-2001
 * SimCoupe : Copyright (c) Allan Skillman 1996.
 *
 * (start/stop)watch emulator timing code (c) 1994 Ian Collier.
 *
 * File util.c - contains various common utility functions
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <conio.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <ctype.h>

#include "mem.h"
#include "util.h"
#include "resource.h"
#include "version.h"
#include "video.h"


//emulator timing variables
static struct timeval watch;
static unsigned long millisec;
static int stopped;


//list of functions to be called when shutting down
static struct on_quit_rec {
  void (*f)(void);
  struct on_quit_rec *next;
  char name[100];
} *on_quit_list;


char log_path[1000]; //full path name of log file


void OnQuit(void (*fn)(void),const char *name) {
  struct on_quit_rec *p;

  p=(struct on_quit_rec*)malloc(sizeof(struct on_quit_rec));
  if(p) {
    simcoupe_mesg(SIMCOUPE_INFO,"registering %s",name);
    p->f = fn;
    p->next = on_quit_list;
    strncpy(p->name,name,sizeof(p->name));
    on_quit_list=p;
  } else {
    simcoupe_mesg(SIMCOUPE_FATAL,"malloc failed in OnQuit()!");
    exit(1);
  }
}


void Quit(int status) {
//
//exit from emulator
//
  struct on_quit_rec *p = on_quit_list;

  VideoClose();

  simcoupe_mesg(SIMCOUPE_INFO,"Quit: status=%i",status);

  while (p) {
    simcoupe_mesg(SIMCOUPE_INFO,"Quit: %s",p->name);
    p->f();
    p=p->next;
  }

//  simcoupe_mesg(SIMCOUPE_INFO,"Quit: OnQuit() list done");

  show_credits();
//  printf("Exiting....\n");

  simcoupe_mesg(SIMCOUPE_INFO,"Quit: exiting application");

  exit(status);
}


void simcoupe_mesg(int error_level,...) {
//
//log-message
//
	va_list args;
	char *fmt;
  FILE *logfp;

	//soubor vzdy otevre a hned zavre
	logfp=fopen(log_path,"a");

	va_start(args, error_level);
	fmt=(char*)va_arg(args,char*);

	if(logfp) {
		if (error_level == SIMCOUPE_WARN)	fprintf(logfp, "warning: ");
		else if (error_level == SIMCOUPE_ERR) fprintf(logfp, "error: ");
//		else if (error_level == SIMCOUPE_INFO) fprintf(logfp, "information: ");
		else if (error_level == SIMCOUPE_FATAL)	fprintf(logfp, "fatal: ");
	
		vfprintf(logfp, fmt, args);
		fputc('\n', logfp);

    fclose(logfp);
	}
	if(error_level==SIMCOUPE_FATAL)	Quit(1);
}


void init_log(void) {
  sprintf(log_path,"%s/%s",exe_path,cfg.log);
  remove(log_path);
} 


void startwatch(int flag) {
//
//emulator timing functions - from xz80 (util.c) by Ian Collier
//
//entry: flag = 0 -> reset stopwatch; 1 -> carry on
//
 if(!flag)millisec=0;
 gettimeofday(&watch,(struct timezone *)0);
 stopped=0;
}


unsigned long stopwatch() {
  unsigned long oldtime;
  unsigned long newtime;
  if(stopped)return millisec;
  oldtime=watch.tv_sec*1000+watch.tv_usec/1000;
  gettimeofday(&watch,(struct timezone *)0);
  newtime=watch.tv_sec*1000+watch.tv_usec/1000;
  stopped=1;
  return millisec+=(newtime-oldtime);
}


void show_credits(void) {
  textattr(0x1e);
  cprintf(" ASCD %s ",VERSION_TEXT);
  textattr(7);
  cprintf("\r\nCopyright (c) Aley Keprt 1998-2002\r\n"
          "http://get.to/aley, e-mail: aley@keprt.cz\r\n\r\n"
          "Based on: SimCoupe 0.79 - (c) Allan J.Skillman & Aley Keprt 1996-1999\r\n"
          "          SAAemu 0.61.2 - (c) Aley Keprt 1996-1999\r\n"
          "          zlib 1.1.3 - (c) Jean-loup Gailly & Mark Adler 1995-1998\r\n"
          "          Mame 0.37 beta 16 - (c) Nicola Salmoria and The MAME Team 1997-2001\r\n"
          "\r\n");
  textattr(7);
}

